<?php

declare(strict_types=1);

namespace Drupal\pdf_reducer;

use Drupal\Component\Utility\Random;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\file\Entity\File;
use Drupal\file\FileInterface;
use GravityMedia\Ghostscript\Enum\PdfSettings;
use GravityMedia\Ghostscript\Ghostscript;
use Symfony\Component\Process\Process;

/**
 *
 */
class PdfReducer {

  public function __construct(protected FileSystemInterface $fileSystem, protected EntityTypeManagerInterface $entityTypeManager) {}

  /**
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Exception
   */
  public function reduce(FileInterface $file): FileInterface {
    $ghostscript = new Ghostscript([
      'quiet' => TRUE,
    ]);
    $dir = $this->fileSystem->getTempDirectory() . '/pdf_reducer/';
    $this->fileSystem->prepareDirectory($dir, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
    $destination = $dir . (new Random())->machineName();
    $device = $ghostscript->createPdfDevice($destination);
    // After some tests, it appears this setting is good enough.
    $device->setPdfSettings(PdfSettings::EBOOK);
    $uri = $file->getFileUri();
    $process = $device->createProcess($uri);
    $process->run(function ($type, $buffer) {
      if ($type === Process::ERR) {
        throw new \RuntimeException($buffer);
      }
      print $buffer;
    });
    $new_file = File::create([
      'uri' => $destination,
      'status' => 0,
    ]);
    // This will get the filesize, mimetype,...
    $new_file->preSave($this->entityTypeManager->getStorage('file'));
    return $new_file;
  }

}
